/*
 * tnmSyslog.c --
 *
 *	A command to access a system logging facility.
 *
 * Copyright (c) 1993-1996 Technical University of Braunschweig.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "tnmInt.h"
#include "tnmPort.h"

TnmTable tnmLogTable[] = {
    { TNM_LOG_EMERG,	"emergency" },
    { TNM_LOG_ALERT,	"alert" },
    { TNM_LOG_CRIT,	"critical" },
    { TNM_LOG_ERR,	"error" },
    { TNM_LOG_WARNING,	"warning" },
    { TNM_LOG_NOTICE,	"notice" },
    { TNM_LOG_INFO,	"info" },
    { TNM_LOG_DEBUG,	"debug" },
    { 0, NULL }
};


/*
 *----------------------------------------------------------------------
 *
 * Tnm_SyslogCmd --
 *
 *	This procedure is invoked to process the "syslog" command.
 *	See the user documentation for details on what it does.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int
Tnm_SyslogCmd(clientData, interp, argc, argv)
    ClientData clientData;
    Tcl_Interp *interp;
    int argc;
    char **argv;
{
    int level;

    if (argc != 3) {
	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
			 " level message\"", (char *) NULL);
	return TCL_ERROR;
    }

    level = TnmGetTableKey(tnmLogTable, argv[1]);

    if (level < 0) {
	Tcl_AppendResult(interp, "bad level \"", argv[1], 
			 "\": should be ", TnmGetTableValues(tnmLogTable),
			 (char *) NULL);
	return TCL_ERROR;
    }

    return TnmWriteLogMessage(interp, level, argv[2]);
}
